// Einfacher Client fr den HelloWorldService
//package helloworld;

/**
 * <p>Title: Verteilte Systeme</p>
 * <p>Description: Jini</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: HSR</p>
 * @author J.M.Joller
 * @version 1.0
 */

import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceTemplate;
import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;

public class HelloWorldClient implements Runnable {
    protected ServiceTemplate template;
    protected LookupDiscovery disco;
    private boolean lookup_successful=false;

    // Innere Klasse implementiert DiscoveryListener
    class Listener implements DiscoveryListener {
        public void discovered(DiscoveryEvent ev) {
        //
          System.out.println("[HellOWorldClient$Listener]discovered()");
          ServiceRegistrar[] newregs = ev.getRegistrars();

          //
          int nr_services = newregs.length;
          if (nr_services>1) System.out.println("[HellOWorldClient$Listener.discovered()]Es wurden "+newregs.length+" Services gefunden");
          else if (nr_services==1) System.out.println("[HellOWorldClient$Listener.discovered()]Es wurde "+newregs.length+" Service gefunden");
          else System.out.println("[HellOWorldClient$Listener.discovered()]Es wurde kein Service gefunden");

          //
          for (int i=0 ; i<newregs.length ; i++) {
            System.out.println("[HellOWorldClient$Listener.discovered()]Service : "+newregs[i]);
            lookForService(newregs[i]);
          }
        }
        public void discarded(DiscoveryEvent ev) {
        }
    }

    public HelloWorldClient() throws IOException {
        Class[] types = { HelloWorldServiceInterface.class };
        for (int k=0; k<types.length; k++)
        System.out.println("[HellOWorldClient.HelloWorldClient()]types["+k+"] "+types[k]);

        System.out.println("[HellOWorldClient.HelloWorldClient()]ServiceTemplate()");
        template = new ServiceTemplate(null, types, null);
        System.out.println("[HellOWorldClient.HelloWorldClient()]ServiceTemplate : "+template);
        System.out.println("[HellOWorldClient.HelloWorldClient()]ServiceTemplate.serviceID : "+template.serviceID);
        System.out.println("[HellOWorldClient.HelloWorldClient()]ServiceTemplate.serviceTypes[0] : "+template.serviceTypes[0]);

        // Security Manager (RMI) setzen
        System.out.println("[HellOWorldClient.HelloWorldClient()]setSecurityManager()");
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(
                    new RMISecurityManager());
        }

        // Public Group durchsuchen
        System.out.println("[HellOWorldClient.HelloWorldClient()]LookupDiscovery()");
		disco = new LookupDiscovery(new String[] {""});
		System.out.println("[HellOWorldClient.HelloWorldClient()]LookupDiscovery() : "+disco);

        // Listener installieren
        System.out.println("[HellOWorldClient.HelloWorldClient()]LookupDiscovery.addDiscoveryListener()");
        disco.addDiscoveryListener(new Listener());
    }

    // neuer Lookup Service
    // Suche Proxies, welche das
    // HelloWorldServiceInterface implementieren

    protected Object lookForService(ServiceRegistrar lusvc) {
        Object o = null;

        System.out.println("[HellOWorldClient]lookForService()");
        try {
            System.out.println("[HellOWorldClient.lookForService()]template - "+template);
            System.out.println("[HellOWorldClient.lookForService()]template.serviceID - "+template.serviceID);
            System.out.println("[HellOWorldClient.lookForService()]template.serviceType[0] - "+template.serviceTypes[0]);
            System.out.println("[HellOWorldClient.lookForService()]template.attributeSetTemplates() - "+template.attributeSetTemplates);
            o = lusvc.lookup(template);
        } catch (RemoteException ex) {
            System.err.println("\t\t\t=>[RemoteException]Fehler beim Lookup: \n\t\t\t" + ex.getMessage()+"; \n\t\t\t=>vermutliche Fehlerursache: CODEBASE" );
            return null;
        }

        if (o == null) {
            System.err.println("\t\t\t=>Es wurde kein passender Service gefunden : "+o);
            return null;
        }

        System.out.println("\t\t\t=>Es wurde ein passender Service gefunden.");
        System.out.println("\t\t\t=>Und hier seine Message an die Welt: " +
        ((HelloWorldServiceInterface)o).getMessage());
        lookup_successful = true;
        return o;
    }

    // Dummy Threads
    // beschftigt die JVM whrend dem Discovery Prozess

    public void run() {
        System.out.println("[HelloWorldClient]run()");
        int loop=0;
        while ((!lookup_successful)&&(loop<5)) {
            try {
		        System.out.println("[HelloWorldClient.run]sleep()");
		        loop++;
                Thread.sleep(10000);
            } catch (InterruptedException ex) {
            }
        }
    }

    // Kreiere einen HelloWorldClient und starte dessen Thread
    public static void main(String args[]) {
      System.out.println("[HelloWorldClient]main()");
      try {
        System.out.println("[HelloWorldClient.main()]HelloWorldClient()");
        HelloWorldClient hwc = new HelloWorldClient();
	System.out.println("[HelloWorldClient.main()]Thread.start()");
        new Thread(hwc).start();
        } catch (IOException ex) {
          System.out.println("Fehler beim Kreieren des Client: " +
          ex.getMessage());
        }
    }
}
