// Hello World Grundversion

//package jini.helloworld;

import net.jini.discovery.DiscoveryListener;
import net.jini.discovery.DiscoveryEvent;
import net.jini.discovery.LookupDiscovery;
import net.jini.core.lookup.ServiceItem;
import net.jini.core.lookup.ServiceRegistrar;
import net.jini.core.lookup.ServiceRegistration;
import java.util.HashMap;
import java.io.IOException;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.RMISecurityManager;
import java.util.*;

// Proxy Objekt
// Der Client ldt diese Klasse ber das Netz,
// als serialisiertes Objekt,
// welches das Interface HelloWorldServiceInterface implementiert

class HelloWorldServiceProxy implements Serializable,
    HelloWorldServiceInterface {
    public HelloWorldServiceProxy() {
	    System.out.println("[HelloWorldServiceProxy]HelloWorldProxy()");
    }
    public String getMessage() {
	    System.out.println("[HelloWorldServiceProxy.HelloWorldProxy()]getMessage()");
        return "Hello, world!";
    }
}

// HelloWorldService ist die "wrapper" Klasse
// Diese publiziert Service Items.

public class HelloWorldService implements Runnable {
    // 10 Minuten leases
    protected final int LEASE_TIME = 10 * 60 * 1000;
    protected HashMap registrations = new HashMap();
    protected ServiceItem item;
    protected LookupDiscovery disco;

    // Innerere Klasse : Listener fr Discovery Events
    class Listener implements DiscoveryListener {
        // Diese wird aufgerufen, falls ein neuer
        // Lookup Service gefunden wird.
        public void discovered(DiscoveryEvent ev) {
            System.out.println("[HelloWorldService$Listener.discovered()]Es wurde ein neuer Lookup Service entdeckt!");
            ServiceRegistrar[] newregs = ev.getRegistrars();
            for (int i=0 ; i<newregs.length ; i++) {
                if (!registrations.containsKey(newregs[i])) {
		            System.out.println("[HelloWorldService$Listener.discovered()]Eintrag in den Lookup Services! - "+newregs[i]);
                    registerWithLookup(newregs[i]);
                }
            }
        }

        // Methode zum expliziten entfernen der
        // Lookup Dienst Information eines Dienstes.

        // Falls ein Lookup Dienst einmal gefunden wurde
        // wird davon ausgegangen, dass er ewig aktiv bleibt.

        public void discarded(DiscoveryEvent ev) {
            System.out.println("[HelloWorldService$Listener.discarded()]Entfernen von Lookup Services!");
            ServiceRegistrar[] deadregs = ev.getRegistrars();
            for (int i=0 ; i<deadregs.length ; i++) {
                registrations.remove(deadregs[i]);
            }
        }
    }

    public HelloWorldService() throws IOException {
        System.out.println("[HelloWorldService]HelloWorldService()");
        item = new ServiceItem(null, createProxy(), null);

        // Security Manager setzen (RMI)
        System.out.println("[HelloWorldService.HelloWorldService()]setSecurityManager()");
        if (System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }

        // Suchen der"public" Group
        // Dieser entspricht ein leerer String

        System.out.println("[HelloWorldService.HelloWorldService()]LookupDiscovery()");
        disco = new LookupDiscovery(new String[] { "" });

        // Installiere einen Listener.
        System.out.println("[HelloWorldService.HelloWorldService()]addListener()");
        disco.addDiscoveryListener(new Listener());
    }

    protected HelloWorldServiceInterface createProxy() {
        System.out.println("[HelloWorldService]createProxy()");
        return new HelloWorldServiceProxy();
    }

    // Nun folgt ein Remote Call
    // Diese knnen dauern....
    // Deswegen wird der Thread schlafend gelegt
    synchronized void registerWithLookup(ServiceRegistrar
                                         registrar) {
        System.out.println("[HelloWorldService]registerWithLookup()");
        ServiceRegistration registration = null;

        try {
            registration = registrar.register(item, LEASE_TIME);
        } catch (RemoteException ex) {
            System.out.println("\t\t\t[RemoteException]Registrierung schlug fehl: " + ex.getMessage());
            return;
        }

        // Ist dies die erste Registrierung?
        // Falls ja: Service-ID abspeichern
        System.out.println("[HelloWorldService.registerWithLookup()]getServiceID()");
        if (item.serviceID == null) {
            item.serviceID = registration.getServiceID();
            System.out.println("\t\t\tServiceID : " + item.serviceID);
            System.out.println("\t\t\tService   : " + item.service);
        }

        registrations.put(registrar, registration);
    }

    // Schlaf-Thread
    // ... damit die VM den Prozess nicht rausschmeisst
    public void run() {
        System.out.println("[HelloWorldService]run()");
        int loop=0;
        boolean maxWait=true;
        while (maxWait) {
			loop++;
			maxWait=loop<10;
	        System.out.println("[HelloWorldService.run()]sleep() "+(new Date().toGMTString()));
            try {
                Thread.sleep(10000);
            } catch (InterruptedException ex) {
            }
        }
    }

    // HelloWorldService kreieren und starten
    // des Threads
    public static void main(String args[]) {
		System.out.println("[HelloWorldService]main()");
        try {
			System.out.println("[HelloWorldService.main()]HelloWorldService()");
            HelloWorldService hws =
                new HelloWorldService();
			System.out.println("[HelloWorldService.main()]Thread.start()");
            new Thread(hws).start();
        } catch (IOException ex) {
            System.out.println("\t\t\t[IOException]Der Service konnte nicht kreiert werden: " +
                               ex.getMessage());
        }
    }
}
